use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007006
use Test::Spelling 0.17;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
49699333
Aas
Adam
Alders
Alders'
Alex
Alexey
Andreas
Bill
Bron
Burke
Chrysostomos
DAVIDRW
Daniel
Date
David
Denaxas
FWILES
Father
Finch
Froehlich
Gavin
Gisle
Gondwana
Graeme
Grossmann
HTTP
Hanak
Hans
Hay
Hedlund
Hukins
Ian
Jacob
James
Josef
Kapranoff
Karaban
Kennedy
Kilgore
Koenig
Kyle
Lipcon
Ludlam
Mann
Mark
Michal
Mike
Olaf
Ondrej
Peter
Peters
Philip
Rabbitson
Raspass
Rezic
Robert
Rolf
Rothenberg
Schilli
Sean
Sjogren
Skyttä
Slaven
Spiros
Steve
SteveHay
Stone
Stosberg
TZ
Thompson
Todd
Tom
Tony
Toru
Tourbin
Ville
Wheeler
Wright
YYYY
Yamaguchi
Yuri
Zefram
adamk
amir
amire80
andreas
asjo
at
brong
david
davidrw
denaxas
dependabot
dot
gisle
github
gpeters
hfroehlich
iank
jefflee
john9art
jraspass
ka
kyle
lib
ludlam
mark
michal
mschilli
murphy
olaf
ondrej
perl
phrstbrn
pnull
rg
ribasushi
ruff
sasao
sburke
shaohua
slaven
sprout
talby
tech
todd
tom
uid39246
ville
waif
wfmann
zefram
zigorou
Špaček
